@@ -188,11 +188,41 @@ class LoadMaster(HttpClient):
     def build_virtual_service(self, service):
         """Create a VirtualService instance with standard defaults"""
         is_sub_vs = True if int(service.get('MasterVSID', 0)) != 0 else False
-        virt_serv = VirtualService(self.access_info,
-                                   service.get('VSAddress'),
-                                   service.get('VSPort'),
-                                   service.get('Protocol'),
-                                   is_sub_vs=is_sub_vs)
+        if is_sub_vs:
+            response = self._get("/showvs", {"vs": service.get('MasterVSID')})
+            data = get_data(response)
+            subvs_id_list, subvs_data = get_sub_vs_list_from_data(data)
+            virt_serv = VirtualService(self.access_info, service.get('Index'),
+                                       is_sub_vs=True)
+            virt_serv.subvs_data = subvs_data[service.get('Index')]
+        else:
+            response = self._get("/listvs")
+            data = get_data(response)
+            virt_serv = VirtualService(self.access_info,
+                                       service.get('VSAddress'),
+                                       service.get('VSPort'),
+                                       service.get('Protocol'),
+                                       is_sub_vs=False)
+            virt_serv.subvs_entries = []
+            for key, value in data.items():
+                if key == "VS":
+                    if not isinstance(value, list):
+                        value = [value]
+                    vs_index = service.get('Index')
+                    for vs in value:
+                        # add subvs to parent vs
+                        if vs['MasterVSID'] == vs_index:
+                            subvs = VirtualService(self.access_info,
+                                                   vs['Index'])
+                            subvs.populate_default_attributes(vs)
+                            subvs_api_entries = service.get("SubVS", [])
+                            if not isinstance(subvs_api_entries, list):
+                                subvs_api_entries = [subvs_api_entries]
+                            for subvs_api in subvs_api_entries:
+                                if subvs_api["VSIndex"] == subvs.index:
+                                    subvs.subvs_data = subvs_api
+
+                            virt_serv.subvs_entries.append(subvs)
         virt_serv.populate_default_attributes(service)
         return virt_serv
 
@@ -369,6 +399,7 @@ class KempBaseObjectModel(object):
     # blacklist attributes that shouldn't be pushed to the loadmaster.
     _API_IGNORE = (
         "ip_address", "endpoint", "rsindex", "vsindex", "index", "status",
+        "subvs_data", "subvs_entries",
     )
 
     def __repr__(self):
@@ -416,6 +447,8 @@ class VirtualService(HttpClient, KempBaseObjectModel):
         """
         self.index = None  # to avoid AttributeErrors later
         self._is_sub_vs = is_sub_vs
+        self.subvs_data = None
+        self.subvs_entries = []
         if not is_sub_vs:
             # Skip validation when it is a subvs as they do not have ip/port
             self.vs = ip_or_index
@@ -426,6 +459,7 @@ class VirtualService(HttpClient, KempBaseObjectModel):
             validate_protocol(prot)
         else:
             self.index = ip_or_index
+            self.vs = ip_or_index
 
         try:
             self.endpoint = loadmaster_info["endpoint"]
@@ -476,16 +510,21 @@ class VirtualService(HttpClient, KempBaseObjectModel):
                 # Hell, thy name be subvs
                 response = self._get("/showvs", self._get_base_parameters())
                 data = get_data(response)
-                existing_subvs_entries = get_sub_vs_list_from_data(data)
+                existing_subvs_entries = get_sub_vs_list_from_data(data)[0]
                 params = self._get_base_parameters()
                 params["createsubvs"] = ""
                 response = self._get("/modvs", params)
                 data = get_data(response)
-                new_subvs_entries = get_sub_vs_list_from_data(data)
+                new_subvs_entries, subvs_data = get_sub_vs_list_from_data(data)
                 s = set(existing_subvs_entries)
                 created_subvs_id = [x for x in new_subvs_entries if x not in s]
                 newly_created_vs_params = {"vs": created_subvs_id}
+                self.subvs_data = subvs_data[created_subvs_id[0]]
                 response = self._get("/showvs", newly_created_vs_params)
+                #data = get_data(response)
+                #subvs = VirtualService(self.access_info, created_subvs_id[0])
+                #subvs.populate_default_attributes(data)
+                #self.subvs_entries.append(subvs)
             else:
                 response = self._get("/addvs", self.to_api_dict())
         else:
